package traceroute

import (
	"errors"
	"net"
)

func resolveDNS(hostname string) (*address, error) {

	ips, err := net.LookupIP(hostname)
	if err != nil {
		return nil, err
	}

	var ipV4 net.IP
	for _, ip := range ips {
		if len(ip) == net.IPv6len {
			return &address{ip: ip, version: addressV6}, nil
		} else if len(ip) == net.IPv4len {
			ipV4 = ip
		}
	}

	if ipV4 != nil {
		return &address{ip: ipV4, version: addressV4}, nil
	}

	return nil, errors.New("could not find a valid record for " + hostname)

}

// DNS resolution utility function (IPv6/IPv4)
func resolve(hostname string) (*address, error) {
	ip := net.ParseIP(hostname)

	if ip.To4() != nil {
		return &address{ip: ip, version: addressV4}, nil
	}

	if ip.To16() != nil {
		return &address{ip: ip, version: addressV6}, nil
	}

	return resolveDNS(hostname)

}
