package traceroute

import (
	"fmt"
	"testing"
)

func TestResolve(t *testing.T) {

	testCases := []struct {
		name           string
		should         string
		shouldHasError bool
		shouldVersion  int
	}{
		{"schukai.com", "80.147.206.177", false, addressV4},
		{"unsupported", "", true, addressV4},
		{"::ffff:192.0.2.1", "192.0.2.1", false, addressV4}, // ipv6 masked ipv4
		{"127.0.0.1", "127.0.0.1", false, addressV4},
		{"2001:4860:4860::8888", "2001:4860:4860::8888", false, addressV6},
		{"example.com", "2606:2800:220:1:248:1893:25c8:1946", false, addressV6},
	}
	for _, tc := range testCases {

		t.Run(fmt.Sprintf("%s should fail %t", tc.name, tc.shouldHasError), func(t *testing.T) {
			address, err := resolve(tc.name)
			if err != nil {
				if tc.shouldHasError == true {
					return
				}
				t.Errorf("dns query error: %s", err.Error())
			}

			if address.String() != tc.should {
				t.Errorf("query %s should %s and not %s", tc.name, tc.should, address.String())
			}

			if tc.shouldVersion == addressV6 {
				if address.isV6() != true || address.isV4() != false {
					t.Errorf("query %s should version V6", tc.name)
				}
			} else {
				if address.isV6() == true || address.isV4() == false {
					t.Errorf("query %s should version V4", tc.name)
				}
			}


		})

	}

}

func benchmarkResolve(address string, b *testing.B) {
	for n := 0; n < b.N; n++ {
		resolve(address)
	}
}

func BenchmarkResolve1(b *testing.B) { benchmarkResolve("www.google.com", b) }
func BenchmarkResolve2(b *testing.B) { benchmarkResolve("www.schukai.com", b) }
func BenchmarkResolve3(b *testing.B) { benchmarkResolve("2001:4860:4860::8888", b) }
func BenchmarkResolve4(b *testing.B) { benchmarkResolve("unsupported", b) }
func BenchmarkResolve5(b *testing.B) { benchmarkResolve("127.0.0.1", b) }
