package traceroute

import "net"

const (
	unknown   = 0
	addressV4 = 4
	addressV6 = 6
)

func isV4(ip net.IP) bool {
	i := ip.To4()
	if i != nil {
		return true
	}
	return false
}

type address struct {
	ip      net.IP
	version int
}

func (a address) isV4() bool {
	return a.version == addressV4
}
func (a address) isV6() bool {
	return a.version == addressV6
}

func (a address) isUnknow() bool {
	return a.version == unknown
}

func (a address) String() string {
	return a.ip.String()
}

func  (a address) Network() string {
	r:= net.IPAddr{
		IP:   a.ip,
		Zone: "",
	}
	return r.Network()
}

func newAddress(ip net.IP) address {

	if ip == nil {
		return address{
			ip:      nil,
			version: unknown,
		}

	}

	if isV4(ip) {
		return address{
			ip:      ip,
			version: addressV4,
		}
	}

	return address{
		ip:      ip,
		version: addressV6,
	}

}
