package traceroute

import (
	"github.com/creasty/defaults"
	"golang.org/x/net/icmp"
	"golang.org/x/net/ipv4"
	"golang.org/x/net/ipv6"
	"time"
)

// Session is Transaction Struct
type Session struct {
	Destination *address
	Source      *address
	
	CallBack func(result Result)

	Timeout time.Duration `default:"5"` // in seconds
	MaxHops int   `default:"30"`
	
	Mode int

	nextHop  int
	isFinale bool

	ipV6Sock   *ipv6.PacketConn
	ipV4Sock   *ipv4.PacketConn
	icmpEcho   icmp.Message
	readBuffer []byte
}

// NewSession creates a new Session 
func NewSession(destination string) (*Session, error) {

	s := Session{}

	if err := defaults.Set(&s); err != nil {
		return nil, err
	}

	dest, err := resolve(destination)
	if err != nil {
		return nil, err
	}
	s.Destination = dest

	if dest.isV6() {
		s.Mode = addressV6
	} else {
		s.Mode = addressV4
	}

	
	s.Timeout = s.Timeout*time.Second
	//* time.Second

	src, err := getOutboundIP(s.Mode)
	if err!=nil {
		return nil, err
	}
	
	s.Source = src

	return &s, nil

}
